package com.qualityeclipse.favorites.actions;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.actions.ActionDelegate;

import com.qualityeclipse.favorites.FavoritesLog;
import com.qualityeclipse.favorites.FavoritesPlugin;

/**
 * Toggle whether the selected projects are audited.
 */
public class ToggleProjectNatureActionDelegate extends ActionDelegate
      implements IWorkbenchWindowActionDelegate
{
   /**
    * The unique identifier for the nature to be added/removed.
    */
   private static final String NATURE_ID =
      FavoritesPlugin.ID + ".propertiesAuditor";

   /**
    * The currently selected projects
    */
   private final Set projects = new HashSet();

   public void init(IWorkbenchWindow window) {
   }

   /**
    * Cache the currently selected projects and update the action
    * enablement and checked state based upon that seletion.
    */
   public void selectionChanged(IAction action, ISelection selection) {
      updateSelectedProjects(selection);
      if (projects.size() > 0) {
         IProject project = (IProject) projects.iterator().next();
         boolean checked;

         /*
          * Either check that the project has our builder
          *
         checked = PropertiesFileAuditor.hasBuilder(project);
          */
         
         // Or check that the project has our nature
         try {
            checked = project.isOpen() 
               && project.hasNature(NATURE_ID);
         }
         catch (CoreException e) {
            checked = false;
            FavoritesLog.logError(e);
         }
         
         action.setEnabled(true);
         action.setChecked(checked);
      }
      else {
         action.setEnabled(false);
         action.setChecked(false);
      }
   }

   /**
    * Update the list of currently selected projects
    * based upon the current selection
    * 
    * @param selection the current selection
    */
   private void updateSelectedProjects(ISelection selection) {
      projects.clear();
      if (!(selection instanceof IStructuredSelection))
         return;
      for (Iterator iter = ((IStructuredSelection) selection).iterator(); iter.hasNext();) {
         
         // translate the selected object into a project
         
         Object elem = iter.next();
         if (!(elem instanceof IResource)) {
            if (!(elem instanceof IAdaptable))
               continue;
            elem = ((IAdaptable) elem).getAdapter(IResource.class);
            if (!(elem instanceof IResource))
               continue;
         }
         if (!(elem instanceof IProject)) {
            elem = ((IResource) elem).getProject();
            if (!(elem instanceof IProject))
               continue;
         }
   
         projects.add(elem);
      }
   }

   /**
    * Add (or remove) the builder to the selected projects
    * so that the project's plug-in properties are audited.
    */
   public void run(IAction action) {
      for (Iterator iter = projects.iterator(); iter.hasNext();) {
         final IProject project = (IProject) iter.next();

         /*
          * The builder can be directly associated with a project
          * without the need for a builder...
          *
          *
         // If the project has the builder,
         // then remove the builder and all audit markers
         
         if (PropertiesFileAuditor.hasBuilder(project)) {
            PropertiesFileAuditor.deleteAuditMarkers(project);
            PropertiesFileAuditor.removeBuilderFromProject(project);
         }
         
         // otherwise add the builder
         // and schedule an audit of the files
         
         else {
            PropertiesFileAuditor.addBuilderToProject(project);
            new Job("Properties File Audit") {
               protected IStatus run(IProgressMonitor monitor) {
                  try {
                     project.build(
                        PropertiesFileAuditor.FULL_BUILD,
                        PropertiesFileAuditor.BUILDER_ID,
                        null,
                        monitor);
                  }
                  catch (CoreException e) {
                     FavoritesLog.logError(e);
                  }
                  return Status.OK_STATUS;
               }
            }.schedule();
         }
         */
         
         // Or the a nature can be used to associate a project with a builder
         
         // Cannot modify closed projects.
         if (!project.isOpen())
            continue;
         
         // Get the description.
         IProjectDescription description;
         try {
            description = project.getDescription();
         }
         catch (CoreException e) {
            FavoritesLog.logError(e);
            continue;
         }
         
         // Toggle the nature.
         List newIds = new ArrayList();
         newIds.addAll(Arrays.asList(description.getNatureIds()));
         int index = newIds.indexOf(NATURE_ID);
         if (index == -1)
            newIds.add(NATURE_ID);
         else
            newIds.remove(index);
         description.setNatureIds(
            (String[]) newIds.toArray(new String[newIds.size()]));
   
         // Save the description.
         try {
            project.setDescription(description, null);
         }
         catch (CoreException e) {
            FavoritesLog.logError(e);
         }
      }
   }
}
