package com.qualityeclipse.favorites.editors;

import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.ObjectUndoContext;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ICellEditorListener;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IEditorActionBarContributor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.ide.IGotoMarker;
import org.eclipse.ui.operations.RedoActionHandler;
import org.eclipse.ui.operations.UndoActionHandler;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.texteditor.ITextEditor;

import com.qualityeclipse.favorites.FavoritesLog;

/* $codepro.preprocessor.if version >= 3.1 $ */
import org.eclipse.swt.widgets.TreeColumn;
/* $codepro.preprocessor.elseif version < 3.1 $
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.jface.viewers.TableTreeViewer;
$codepro.preprocessor.endif $ */

/**
 * Editor for modifying *.property files containing content in the
 * typical key=value format
 */
public class PropertiesEditor extends MultiPageEditorPart
{
   public static final String VALUE_COLUMN_ID = "Value";
   public static final String KEY_COLUMN_ID = "Key";
   
   private TextEditor textEditor;

   /* $codepro.preprocessor.if version >= 3.1 $ */
   private TreeViewer treeViewer;
   private TreeColumn keyColumn;
   private TreeColumn valueColumn;
   /* $codepro.preprocessor.elseif version < 3.1 $
   private TableTreeViewer treeViewer;
   private TableColumn keyColumn;
   private TableColumn valueColumn;
   $codepro.preprocessor.endif $ */
   
   private PropertiesEditorContentProvider treeContentProvider;

   private boolean isAltPressed;
   private boolean isPageModified;
   
   private RemovePropertiesAction removeAction;

   private UndoActionHandler undoAction;
   private RedoActionHandler redoAction;
   private IUndoContext undoContext;

   /**
    * When a user edits a value, the model generates a change event to
    * notify registered listeners. This change listener receives
    * notification events and updates the tree appropriately.
    */
   private final PropertyFileListener propertyFileListener =
      new PropertyFileListener()
   {
      public void keyChanged(
         PropertyCategory category, PropertyEntry entry
      ) {
         treeViewer.update(entry, new String[] { KEY_COLUMN_ID });
         treeModified();
      }

      public void valueChanged(
         PropertyCategory category, PropertyEntry entry
      ) {
         treeViewer.update(entry, new String[] { VALUE_COLUMN_ID });
         treeModified();
      }

      public void nameChanged(PropertyCategory category) {
         treeViewer.update(category, new String[] { KEY_COLUMN_ID });
         treeModified();
      }

      public void entryAdded(
         PropertyCategory category, PropertyEntry entry
      ) {
         treeViewer.refresh();
         treeModified();
      }

      public void entryRemoved(
         PropertyCategory category, PropertyEntry entry
      ) {
         treeViewer.refresh();
         treeModified();
      }

      public void categoryAdded(PropertyCategory category) {
         treeViewer.refresh();
         treeModified();
      }

      public void categoryRemoved(PropertyCategory category) {
         treeViewer.refresh();
         treeModified();
      }
   };

   /**
    * Initializes this editor with the given editor site and input.
    * This method is automatically called shortly after the part is
    * instantiated, marking the start of the part's lifecycle.
    * In our case, we assert that we are editing a file.
    */
   public void init(IEditorSite site, IEditorInput input)
      throws PartInitException
   {
      if (!(input instanceof IFileEditorInput))
         throw new PartInitException(
            "Invalid Input: Must be IFileEditorInput");
      super.init(site, input);
   }

   /**
    * Creates the Source and Properties pages 
    * of this multi-page editor.
    */
   protected void createPages() {
      createPropertiesPage();
      createSourcePage();
      updateTitle();
      initTreeContent();
      initTreeEditors();
      createActions();
      createContextMenu();
   }

   void createPropertiesPage() {

      /* $codepro.preprocessor.if version >= 3.1 $ */
      treeViewer = new TreeViewer(
            getContainer(), SWT.MULTI | SWT.FULL_SELECTION);

      Tree tree = treeViewer.getTree();
      tree.setHeaderVisible(true);
      
      keyColumn = new TreeColumn(tree, SWT.NONE);
      valueColumn = new TreeColumn(tree, SWT.NONE);

      /* $codepro.preprocessor.elseif version < 3.1 $
      treeViewer = new TableTreeViewer(
            getContainer(), SWT.MULTI | SWT.FULL_SELECTION);
      
      Table table = treeViewer.getTableTree().getTable();
      table.setHeaderVisible(true);
      
      keyColumn = new TableColumn(table, SWT.NONE);
      valueColumn = new TableColumn(table, SWT.NONE);
      
      $codepro.preprocessor.endif $ */

      keyColumn.setText("Key");
      keyColumn.setWidth(150);
      
      valueColumn.setText("Value");
      valueColumn.setWidth(150);

      int index = addPage(treeViewer.getControl());
      setPageText(index, "Properties");
   }

   void createSourcePage() {
      try {
         textEditor = new TextEditor();
         int index = addPage(textEditor, getEditorInput());
         setPageText(index, "Source");
      }
      catch (PartInitException e) {
         FavoritesLog.logError("Error creating nested text editor",e);
      }
   }
   
   /**
    * Update the editor's title based upon the content being edited.
    */
   void updateTitle() {
      IEditorInput input = getEditorInput();
      setPartName(input.getName());
      setTitleToolTip(input.getToolTipText());
   }

   /**
    * Initialize the model behind the editor.
    */
   void initTreeContent() {
      treeContentProvider = new PropertiesEditorContentProvider();
      treeViewer.setContentProvider(treeContentProvider);
      treeViewer.setLabelProvider(new PropertiesEditorLabelProvider());
      
      // Reset the input from the text editors content
      // after the editor initialization has completed.
      treeViewer.setInput(new PropertyFile(""));
      treeViewer.getControl().getDisplay().asyncExec(new Runnable() {
         public void run() {
            updateTreeFromTextEditor();
         }
      });
      treeViewer.setAutoExpandLevel(TreeViewer.ALL_LEVELS);
   }

   /**
    * Initialize the cell editors in the tree
    */
   private void initTreeEditors() {
      treeViewer.setColumnProperties(
         new String[] { KEY_COLUMN_ID, VALUE_COLUMN_ID });

      final TextCellEditor keyEditor =
         new TextCellEditor((Composite) treeViewer.getControl());
      final TextCellEditor valueEditor =
         new TextCellEditor((Composite) treeViewer.getControl());
      
      // Validators to verify user input in the tree cell editors
      
      keyEditor.setValidator(new ICellEditorValidator() {
         public String isValid(Object value) {
            if (((String) value).trim().length() == 0)
               return "Key must not be empty string";
            return null;
         }
      });
      valueEditor.setValidator(new ICellEditorValidator() {
         public String isValid(Object value) {
            return null;
         }
      });
      
      // Listener to display validation error messages in the status area
      
      keyEditor.addListener(new ICellEditorListener() {
         public void applyEditorValue() {
            setErrorMessage(null);
         }
         public void cancelEditor() {
            setErrorMessage(null);
         }
         public void editorValueChanged(
            boolean oldValidState,
            boolean newValidState
         ) {
            setErrorMessage(keyEditor.getErrorMessage());
         }
         void setErrorMessage(String errorMessage) {
            getEditorSite().getActionBars().getStatusLineManager()
               .setErrorMessage(errorMessage);
         }
      });
      
      /* $codepro.preprocessor.if version >= 3.1 $ */
      treeViewer.setCellEditors(
         new CellEditor[] { keyEditor, valueEditor });
      treeViewer.setCellModifier(
         new PropertiesEditorCellModifier(this, treeViewer));
      /* $codepro.preprocessor.endif $ */
      
      // Listener to track the state of the ALT key
      
      treeViewer.getControl().addKeyListener(new KeyListener() {
         public void keyPressed(KeyEvent e) {
            if (e.keyCode == SWT.ALT)
               isAltPressed = true;
            if (e.character == SWT.DEL)
               removeAction.run();
         }
         public void keyReleased(KeyEvent e) {
            if (e.keyCode == SWT.ALT)
               isAltPressed = false;
         }
      });
   }

   /**
    * Create actions for the editor that appear as menu items in the
    * views context menu, as toolbar buttons in the workbenchs
    * toolbar, and as menu items in the workbenchs menu.
    */
   private void createActions() {
      undoContext = new ObjectUndoContext(this);
      undoAction = new UndoActionHandler(getSite(), undoContext);
      redoAction = new RedoActionHandler(getSite(), undoContext);

      ImageDescriptor removeImage = PlatformUI.getWorkbench()
         .getSharedImages().getImageDescriptor(
            ISharedImages.IMG_TOOL_DELETE);
      removeAction =
         new RemovePropertiesAction(
            this, treeViewer, "Remove", removeImage);
   }
   
   /**
    * Create the context menu for the editor. Because contributors can
    * add and remove menu items based on the selection, its contents
    * cannot be determined until just after the user clicks the right
    * mouse button and just before the menu is displayed.
    * @see #fillContextMenu(IMenuManager)
    */
   private void createContextMenu() {
      MenuManager menuMgr = new MenuManager("#PopupMenu");
      menuMgr.setRemoveAllWhenShown(true);
      menuMgr.addMenuListener(new IMenuListener() {
         public void menuAboutToShow(IMenuManager m) {
            PropertiesEditor.this.fillContextMenu(m);
         }
      });
      Control control = treeViewer.getControl();
      Menu menu = menuMgr.createContextMenu(control);
      control.setMenu(menu);
      getSite().registerContextMenu(menuMgr,treeViewer);
   }
   
   /**
    * Dynamically build the context menu content just after the user
    * clicks the right mouse button and just before the menu is displayed.
    * @see #createContextMenu()
    */
   private void fillContextMenu(IMenuManager menuMgr) {
      menuMgr.add(undoAction);
      menuMgr.add(redoAction);
      menuMgr.add(new Separator());
      boolean isEmpty = treeViewer.getSelection().isEmpty();
      removeAction.setEnabled(!isEmpty);
      menuMgr.add(removeAction);
      menuMgr.add(
         new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
   }

   /**
    * Called when the properties page should be updated
    * based upon new input from the source page
    */
   void updateTreeFromTextEditor() {
      PropertyFile propertyFile = (PropertyFile) treeViewer.getInput();
      propertyFile.removePropertyFileListener(propertyFileListener);
      propertyFile = new PropertyFile(
         textEditor
            .getDocumentProvider()
            .getDocument(textEditor.getEditorInput())
            .get());
      treeViewer.setInput(propertyFile);
      propertyFile.addPropertyFileListener(propertyFileListener);
   }
   
   /**
    * Called when the source page should be updated
    * based upon new input from the properties page.
    */
   void updateTextEditorFromTree() {
      textEditor
         .getDocumentProvider()
         .getDocument(textEditor.getEditorInput())
         .set(((PropertyFile) treeViewer.getInput()).asText());
   }
   
   /**
    * Called when a cell editor has modified the editor model.
    */
   public void treeModified() {
      boolean wasDirty = isDirty();
      isPageModified = true;
      if (!wasDirty)
         firePropertyChange(IEditorPart.PROP_DIRTY);
   }

   /**
    * Answer true if the ALT key is pressed
    * and the cell editor should be opened.
    */
   public boolean shouldEdit() {
      if (!isAltPressed)
         return false;
      // Must reset this value here because if an editor
      // is opened, we dont get the Alt key up event.
      isAltPressed = false;
      return true;
   }
   
   /**
    * Handles a property change notification from a nested editor. In
    * our case, the <code>isPageModified</code> field is adjusted as
    * appropriate and superclass is called to notify listeners of the
    * change.
    */
   protected void handlePropertyChange(int propertyId) {
      if (propertyId == IEditorPart.PROP_DIRTY)
         isPageModified = isDirty();
      super.handlePropertyChange(propertyId);
   }
   
   /**
    * Returns whether the contents of this editor have changed since
    * the last save operation. If this value changes the editor must
    * fire a property listener event with <code>PROP_DIRTY</code>.
    */
   public boolean isDirty() {
      return isPageModified || super.isDirty();
   }

   /**
    * Notifies this multi-page editor that the page with the given id has been
    * activated. This method is called when the user selects a different tab.
    */
   protected void pageChange(int newPageIndex) {
      switch (newPageIndex) {
         case 0 :
            if (isDirty())
               updateTreeFromTextEditor();
            break;
         case 1 :
            if (isPageModified)
               updateTextEditorFromTree();
            break;
      }
      isPageModified = false;
      
      super.pageChange(newPageIndex);
      
      IEditorActionBarContributor contributor = 
         getEditorSite().getActionBarContributor();
      if (contributor instanceof PropertiesEditorContributor)
         ((PropertiesEditorContributor) contributor)
            .setActivePage(this, newPageIndex);
      
      getOperationHistory().dispose(undoContext, true, true, false);
   }

   public int getActivePage() {
      return super.getActivePage();
   }

   public ITextEditor getSourceEditor() {
      return textEditor;
   }

   public IAction getTreeAction(String workbenchActionId) {
      if (ActionFactory.UNDO.getId().equals(workbenchActionId))
         return undoAction;
      if (ActionFactory.REDO.getId().equals(workbenchActionId))
         return redoAction;
      if (ActionFactory.DELETE.getId().equals(workbenchActionId))
         return removeAction;
      return null;
   }

   /**
    * Asks this part to take focus within the workbench.
    * In our case, focus is redirected to the appropriate
    * control based upon what page is currently selected.
    */
   public void setFocus() {
      switch (getActivePage()) {
         case 0:
            treeViewer.getControl().setFocus();
            break;
         case 1:
            textEditor.setFocus();
            break;
      }
   }
   
   /**
    * Sets the cursor and selection state for an editor to 
    * reveal the position of the given marker.
    */
   public void gotoMarker(IMarker marker) {
      setActivePage(1);
      ((IGotoMarker) getAdapter(IGotoMarker.class))
         .gotoMarker(marker);
   }
   
   /**
    * Returns whether the "Save As" operation is supported by this part.
    */
   public boolean isSaveAsAllowed() {
      return true;
   }

   /**
    * Saves the contents of this part. If the save is successful, the
    * part should fire a property changed event reflecting the new
    * dirty state (<code>PROP_DIRTY</code> property).
    */
   public void doSave(IProgressMonitor monitor) {
      if (getActivePage() == 0 && isPageModified)
         updateTextEditorFromTree();
      isPageModified = false;
      textEditor.doSave(monitor);
   }

   /**
    * Saves the contents of this part to another object. Implementors
    * are expected to open a "Save As" dialog where the user will be
    * able to select a new name for the contents. After the selection
    * is made, the contents should be saved to that new name. If the
    * save is successful, the part fires a property changed event
    * reflecting the new dirty state (<code>PROP_DIRTY</code>
    * property).
    */
   public void doSaveAs() {
      if (getActivePage() == 0 && isPageModified)
         updateTextEditorFromTree();
      isPageModified = false;
      textEditor.doSaveAs();
      setInput(textEditor.getEditorInput());
      updateTitle();
   }

   /**
    * Get the operation history from the workbench.
    * This is also known as the undo/redo manager
    */
   public IOperationHistory getOperationHistory() {
      
      // The workbench provides its own undo/redo manager
      //return PlatformUI.getWorkbench()
      //   .getOperationSupport().getOperationHistory();
      
      // which, in this case, is the same as the default undo manager
      return OperationHistoryFactory.getOperationHistory();
   }

   /**
    * Answer an undo context used to "tag" operations as being
    * applicable to this editor. The undo context is used to filter
    * the history of operations available for undo or redo so that
    * only operations appropriate for a given undo context are shown
    * when the application is presenting that context.
    */
   public IUndoContext getUndoContext() {
      
      // For workbench-wide operations, we should return
      //return PlatformUI.getWorkbench()
      //   .getOperationSupport().getUndoContext();
      
      // but our operations are all local, so return our own content
      return undoContext;
   }
}
