package com.qualityeclipse.favorites.editors;

import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.LabelRetargetAction;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.part.EditorActionBarContributor;
import org.eclipse.ui.texteditor.ITextEditor;

/**
 * A editor action bar contributor defines the actions for one or more
 * editors. Within the workbench there may be more than one open
 * editor of a particular type. For instance, there may be 1 or more
 * open Java Editors. To avoid the creation of duplicate actions and
 * action images the editor concept has been split into two. An action
 * contributor is responsable for the creation of actions. The editor
 * is responsible for action implementation. Furthermore, the
 * contributor is shared by each open editor. As a result of this
 * design there is only 1 set of actions for 1 or more open editors.
 */
public class PropertiesEditorContributor
   extends EditorActionBarContributor
{
   private static final String[] WORKBENCH_ACTION_IDS = {
      ActionFactory.DELETE.getId(),
      ActionFactory.UNDO.getId(),
      ActionFactory.REDO.getId(),
      ActionFactory.CUT.getId(),
      ActionFactory.COPY.getId(),
      ActionFactory.PASTE.getId(),
      ActionFactory.SELECT_ALL.getId(),
      ActionFactory.FIND.getId(),
      IDEActionFactory.BOOKMARK.getId(),
   };
   private static final String[] TEXTEDITOR_ACTION_IDS = {
      ActionFactory.DELETE.getId(),
      ActionFactory.UNDO.getId(),
      ActionFactory.REDO.getId(),
      ActionFactory.CUT.getId(),
      ActionFactory.COPY.getId(),
      ActionFactory.PASTE.getId(),
      ActionFactory.SELECT_ALL.getId(),
      ActionFactory.FIND.getId(),
      IDEActionFactory.BOOKMARK.getId(),
   };

   private LabelRetargetAction retargetRemoveAction =
      new LabelRetargetAction(ActionFactory.DELETE.getId(), "Remove");
   
   /**
    * Initializes this contributor, which is expected to add
    * contributions as required to the given action bars and global
    * action handlers. In our case, we add the retarget actions as
    * part listeners.
    */
   public void init(IActionBars bars, IWorkbenchPage page) {
      super.init(bars, page);
      page.addPartListener(retargetRemoveAction);
   }

   /**
    * Contributes to the given menu.
    */
   public void contributeToMenu(IMenuManager menuManager) {
      IMenuManager menu = new MenuManager("Property Editor");
      menuManager.prependToGroup(
         IWorkbenchActionConstants.MB_ADDITIONS,
         menu);
      menu.add(retargetRemoveAction);
   }

   /**
    * Contributes to the given tool bar.
    */
   public void contributeToToolBar(IToolBarManager manager) {
      manager.add(new Separator());
      manager.add(retargetRemoveAction);
   }
   
   /**
    * Disposes this contributor. 
    */
   public void dispose() {
      getPage().removePartListener(retargetRemoveAction);
      super.dispose();
   }


   /**
    * Sets the active editor for the contributor. Generally this
    * involves disconnecting from the old editor, connect to the new
    * editor, and update the actions to reflect the new editor. In our
    * case, we are setting global action hooks which are automatically
    * disconnected the next time they are set.
    */
   public void setActiveEditor(IEditorPart part) {
      PropertiesEditor editor = (PropertiesEditor) part;
      setActivePage(editor, editor.getActivePage());
   }
   
   public void setActivePage(
      PropertiesEditor editor,
      int pageIndex
   ) {
      IActionBars actionBars = getActionBars();
      if (actionBars != null) {
         switch (pageIndex) {
            case 0 :
               hookGlobalTreeActions(editor, actionBars);
               break;
            case 1 :
               hookGlobalTextActions(editor, actionBars);
               break;
         }
         actionBars.updateActionBars();
      }
   }
   
   private void hookGlobalTreeActions(
      PropertiesEditor editor,
      IActionBars actionBars
   ) {
      for (int i = 0; i < WORKBENCH_ACTION_IDS.length; i++)
         actionBars.setGlobalActionHandler(
            WORKBENCH_ACTION_IDS[i],
            editor.getTreeAction(WORKBENCH_ACTION_IDS[i]));
   }
   
   private void hookGlobalTextActions(
      PropertiesEditor editor,
      IActionBars actionBars
   ) {
      ITextEditor textEditor = editor.getSourceEditor();
      for (int i = 0; i < WORKBENCH_ACTION_IDS.length; i++)
         actionBars.setGlobalActionHandler(
            WORKBENCH_ACTION_IDS[i],
            textEditor.getAction(TEXTEDITOR_ACTION_IDS[i]));
   }
}