package com.qualityeclipse.favorites.editors;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;

/**
 * The underlying operation for removing properties that can be
 * executed, undone and redone.
 */
public class RemovePropertiesOperation extends AbstractOperation
{
   private final TreeViewer viewer;
   private final PropertyElement[] elements;
   private PropertyElement[] parents;
   private int[] indexes;

   /**
    * Construct a new operation for removing the specified properties
    * @param viewer TODO
    * @param elements the properties to be removed (not <code>null</code>)
    */
   public RemovePropertiesOperation(TreeViewer viewer, PropertyElement[] elements) {
      super(getLabelFor(elements));
      this.viewer = viewer;
      this.elements = elements;
   }

   /**
    * Execute the operation by removing the specified properties. In
    * the process we record information about the elements being
    * removed so that this operation can be undone. This method is
    * only be called the first time that an operation is executed.
    * 
    * @param monitor the progress monitor (or <code>null</code>) to
    *           use for reporting progress to the user.
    * @param info the IAdaptable (or <code>null</code>) provided by
    *           the caller in order to supply UI information for
    *           prompting the user if necessary. When this parameter
    *           is not <code>null</code>, it should minimally
    *           contain an adapter for the
    *           org.eclipse.swt.widgets.Shell.class.
    * 
    * @return the IStatus of the execution. The status severity should
    *         be set to <code>OK</code> if the operation was
    *         successful, and <code>ERROR</code> if it was not. Any
    *         other status is assumed to represent an incompletion of
    *         the execution.
    * @throws ExecutionException if an exception occurred during
    *            execution.
    */
   public IStatus execute(IProgressMonitor monitor, IAdaptable info)
      throws ExecutionException
   {
      // If a UI context has been provided, 
      // then prompt the user to confirm the operation
      
      if (info != null) {
         Shell shell = (Shell) info.getAdapter(Shell.class);
         if (shell != null) {
            if (!MessageDialog.openQuestion(
               shell, 
               "Remove properties",
               "Do you want to remove the currently selected properties?"
            ))
               return Status.CANCEL_STATUS;
         }
      }
   
      // Perform the operation
      
      return redo(monitor, info);
   }

   /**
    * Redo the operation similar to {@link #execute(IProgressMonitor, IAdaptable)}. 
    * This method is only called after an operation has been undone.
    */
   public IStatus redo(IProgressMonitor monitor, IAdaptable info)
      throws ExecutionException
   {
      // Perform the operation, providing feedback to the user
      // through the progress monitor if one is provided
      
      parents = new PropertyElement[elements.length];
      indexes = new int[elements.length];
      
      if (monitor != null)
         monitor.beginTask("Remove properties", elements.length);
      
      Tree tree = viewer.getTree();
      tree.setRedraw(false);
      try {
         for (int i = elements.length; --i >= 0;) {
            parents[i] = elements[i].getParent();
            PropertyElement[] children = parents[i].getChildren();
            for (int index = 0; index < children.length; index++) {
               if (children[index] == elements[i]) {
                  indexes[i] = index;
                  break;
               }
            }
            elements[i].removeFromParent();
            
            if (monitor != null)
               monitor.worked(1);
         }
      }
      finally {
         tree.setRedraw(true);
      }
      
      if (monitor != null)
         monitor.done();
      
      return Status.OK_STATUS;
   }

   /**
    * Undo the operation perfomed by {@link #execute(IProgressMonitor, IAdaptable)}. 
    * This method is only called after an operation has been executed or redone.
    */
   public IStatus undo(IProgressMonitor monitor, IAdaptable info)
      throws ExecutionException
   {
      Tree tree = viewer.getTree();
      tree.setRedraw(false);
      try {
         for (int i = 0; i < elements.length; i++) {
            if (parents[i] instanceof PropertyCategory)
               ((PropertyCategory) parents[i]).addEntry(indexes[i],
                     (PropertyEntry) elements[i]);
            else
               ((PropertyFile) parents[i]).addCategory(indexes[i],
                     (PropertyCategory) elements[i]);
         }
      }
      finally {
         tree.setRedraw(true);
      }
      return Status.OK_STATUS;
   }

   /**
    * Answer a label for the specified elements to be removed
    * @param elements the properties to be removed (not <code>null</code>)
    */
   private static String getLabelFor(PropertyElement[] elements) {
      if (elements.length == 1) {
         PropertyElement first = elements[0];
         if (first instanceof PropertyEntry) {
            PropertyEntry propEntry = (PropertyEntry) first;
            return "Remove property " + propEntry.getKey();
         }
         if (first instanceof PropertyCategory) {
            PropertyCategory propCat = (PropertyCategory) first;
            return "Remove category " + propCat.getName();
         }
      }
      return "Remove properties";
   }
}
