package com.qualityeclipse.favorites.model;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.XMLMemento;

import com.qualityeclipse.favorites.FavoritesLog;
import com.qualityeclipse.favorites.FavoritesPlugin;

public class FavoritesManager
   implements IResourceChangeListener
{
   private static final String TAG_FAVORITES = "Favorites";
   private static final String TAG_FAVORITE = "Favorite";
   private static final String TAG_TYPEID = "TypeId";
   private static final String TAG_INFO = "Info";
   
   private static FavoritesManager manager;
   private Collection favorites;
   private List listeners = new ArrayList();

   private FavoritesManager() {
      ResourcesPlugin.getWorkspace().addResourceChangeListener (
         this, IResourceChangeEvent.POST_CHANGE);
   }

   ////////////////////////////////////////////////////////////////////////////
   //
   // Singleton
   //
   ////////////////////////////////////////////////////////////////////////////
   
   public static FavoritesManager getManager() {
      if (manager == null)
         manager = new FavoritesManager();
      return manager;
   }
   
   public static void shutdown() {
      if (manager != null) {
         ResourcesPlugin.getWorkspace()
            .removeResourceChangeListener(manager);
         manager.saveFavorites();
         manager = null;
      }
   }

   ////////////////////////////////////////////////////////////////////////////
   //
   // Accessing Favorite Items
   //
   ////////////////////////////////////////////////////////////////////////////

   public IFavoriteItem[] getFavorites() {
      if (favorites == null)
         loadFavorites();
      return (IFavoriteItem[]) favorites
            .toArray(new IFavoriteItem[favorites.size()]);
   }
   
   public IFavoriteItem newFavoriteFor(Object obj) {
      FavoriteItemType[] types = FavoriteItemType.getTypes();
      for (int i = 0; i < types.length; i++) {
         IFavoriteItem item = types[i].newFavorite(obj);
         if (item != null)
            return item;
      }
      return null;
   }
   
   public IFavoriteItem[] newFavoritesFor(Iterator iter) {
      if (iter == null)
         return IFavoriteItem.NONE;
      Collection items = new HashSet(20);
      while (iter.hasNext()) {
         IFavoriteItem item = newFavoriteFor((Object) iter.next());
         if (item != null)
            items.add(item);
      }
      return (IFavoriteItem[]) items.toArray(new IFavoriteItem[items
            .size()]);
   }
   
   public IFavoriteItem[] newFavoritesFor(Object[] objects) {
      if (objects == null)
         return IFavoriteItem.NONE;
      return newFavoritesFor(Arrays.asList(objects).iterator());
   }
   
   public IFavoriteItem existingFavoriteFor(Object obj) {
      if (obj == null)
         return null;
      Iterator iter = favorites.iterator();
      while (iter.hasNext()) {
         IFavoriteItem item = (IFavoriteItem) iter.next();
         if (item.isFavoriteFor(obj))
            return item;
      }
      return null;
   }
   
   public IFavoriteItem[] existingFavoritesFor(Iterator iter) {
      List result = new ArrayList(10);
      while (iter.hasNext()) {
         IFavoriteItem item = existingFavoriteFor(iter.next());
   
         if (item != null)
            result.add(item);
      }
      return (IFavoriteItem[]) result
            .toArray(new IFavoriteItem[result.size()]);
   }
   
   public void addFavorites(IFavoriteItem[] items) {
      if (favorites == null)
         loadFavorites();
      if (favorites.addAll(Arrays.asList(items)))
         fireFavoritesChanged(items, IFavoriteItem.NONE);
   }
   
   public void removeFavorites(IFavoriteItem[] items) {
      if (favorites == null)
         loadFavorites();
      if (favorites.removeAll(Arrays.asList(items)))
         fireFavoritesChanged(IFavoriteItem.NONE, items);
   }
   
   ////////////////////////////////////////////////////////////////////////////
   //
   // Event Handling
   //
   ////////////////////////////////////////////////////////////////////////////

   public void addFavoritesManagerListener(
      FavoritesManagerListener listener
   ) {
      if (!listeners.contains(listener))
         listeners.add(listener);
   }
   
   public void removeFavoritesManagerListener(
      FavoritesManagerListener listener
   ) {
      listeners.remove(listener);
   }
   
   private void fireFavoritesChanged(
      IFavoriteItem[] itemsAdded, IFavoriteItem[] itemsRemoved
   ) {
      FavoritesManagerEvent event = new FavoritesManagerEvent(
         this, itemsAdded, itemsRemoved);
      for (Iterator iter = listeners.iterator(); iter.hasNext();)
         ((FavoritesManagerListener) iter.next())
               .favoritesChanged(event);
   }
   
   ////////////////////////////////////////////////////////////////////////////
   //
   // Persisting favorites
   //
   ////////////////////////////////////////////////////////////////////////////

   private void loadFavorites() {
      favorites = new HashSet(20);
      FileReader reader = null;
      try {
         reader = new FileReader (getFavoritesFile());
         loadFavorites(XMLMemento.createReadRoot(reader));
      }
      catch (FileNotFoundException e) {
         // Ignored... no Favorites items exist yet.
      }
      catch (Exception e) {
         // Log the exception and move on.
         FavoritesLog.logError(e);
      }
      finally {
         try {
            if (reader != null)
               reader.close();
         }
         catch (IOException e) {
            FavoritesLog.logError(e);
         }
      }
   }
   
   private void loadFavorites(XMLMemento memento) {
      IMemento [] children = memento.getChildren(TAG_FAVORITE);
      for (int i = 0; i < children.length; i++) {
         IFavoriteItem item =
            newFavoriteFor(
               children[i].getString(TAG_TYPEID),
               children[i].getString(TAG_INFO));
         if (item != null)
            favorites.add(item);
      }
   }

   public IFavoriteItem newFavoriteFor(String typeId, String info) {
      FavoriteItemType[] types = FavoriteItemType.getTypes();
      for (int i = 0; i < types.length; i++)
         if (types[i].getId().equals(typeId))
            return types[i].loadFavorite(info);
      return null;
   }

   private void saveFavorites() {
      if (favorites == null)
         return;
      XMLMemento memento = XMLMemento.createWriteRoot(TAG_FAVORITES);
      saveFavorites(memento);
      FileWriter writer = null; 
      try {
         writer = new FileWriter(getFavoritesFile());
         memento.save(writer);
      }
      catch (IOException e) {
         FavoritesLog.logError(e);
      }
      finally {
         try {
            if (writer != null)
               writer.close();
         }
         catch (IOException e) {
            FavoritesLog.logError(e);
         }
      }
   }

   private void saveFavorites(XMLMemento memento) {
      Iterator iter = favorites.iterator();
      while (iter.hasNext()) {
         IFavoriteItem item = (IFavoriteItem) iter.next();
         IMemento child = memento.createChild(TAG_FAVORITE);
         child.putString(TAG_TYPEID, item.getType().getId());
         child.putString(TAG_INFO, item.getInfo());
      }
   }

   private File getFavoritesFile() {
      return FavoritesPlugin
         .getDefault()
         .getStateLocation()
         .append("favorites.xml")
         .toFile();
   }

   ////////////////////////////////////////////////////////////////////////////
   //
   // Resource change event handling
   //
   ////////////////////////////////////////////////////////////////////////////

   /**
    * Called by the system when some resource changes are happening,
    * or have already happened. The supplied event gives details. This
    * event object (and the resource delta within it) is valid only
    * for the duration of the invocation of this method.
    */
   public void resourceChanged(IResourceChangeEvent event) {
      final Collection itemsToRemove = new HashSet();
      try {
         event.getDelta().accept(new IResourceDeltaVisitor() {
            public boolean visit(IResourceDelta delta)
               throws CoreException
            {
               if (delta.getKind() == IResourceDelta.REMOVED) {
                  IFavoriteItem item =
                     existingFavoriteFor(delta.getResource());
                  if (item != null)
                     itemsToRemove.add(item);
               }
               return true;
            }
         });
      }
      catch (CoreException ex) {
         FavoritesLog.logError(ex);
      }
      if (itemsToRemove.size() > 0)
         removeFavorites(
            (IFavoriteItem[]) itemsToRemove.toArray(
               new IFavoriteItem[itemsToRemove.size()]));
   }
}
