package com.qualityeclipse.favorites.util;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaModelException;

import com.qualityeclipse.favorites.FavoritesLog;

/**
 * The ExecuteMethodActionDelegate uses ProjectClassLoader to load the
 * selected class into the Favorites plug-in to be executed. This
 * ClassLoader locates the class file using the projects Java build
 * path, reads the class file using standard java.io, and creates the
 * class in memory using the superclass defineClass() method. It is
 * not complete as it only loads source-based classes; loading classes
 * from a JAR file or reference project is left as an exercise for the
 * reader.
 */
public class ProjectClassLoader extends ClassLoader
{
   /**
    * The project for which classes are loaded
    */
   private IJavaProject project;

   /**
    * Construct a new instance for loading classes for a specific
    * project.
    * 
    * @param project the project for which classes are loaded
    */
   public ProjectClassLoader(IJavaProject project) {
      if (project == null || !project.exists() || !project.isOpen())
         throw new IllegalArgumentException("Invalid project");
      this.project = project;
   }

   /**
    * Finds the class with the specified binary name. We override this
    * method to find the class in the associated project.
    * 
    * @param name The binary name of the class
    * @return The resulting Class object
    * @throws ClassNotFoundException If the class could not be found
    */
   protected Class findClass(String name) throws ClassNotFoundException
   {
      byte[] buf = readBytes(name);
      if (buf == null)
         throw new ClassNotFoundException(name);
      return defineClass(name, buf, 0, buf.length);
   }

   /**
    * Locate a class with the specified name and load the bytes from
    * its classfile.
    * 
    * @param name the fully qualified name of the class to be located.
    * @return the bytes or <code>null</code> if not found
    */
   private byte[] readBytes(String name) {
      IPath rootLoc = ResourcesPlugin.getWorkspace().getRoot()
            .getLocation();
      Path relativePathToClassFile = new Path(name.replace('.', '/')
            + ".class");
      IClasspathEntry[] entries;
      IPath outputLocation;
      try {
         entries = project.getResolvedClasspath(true);
         outputLocation = rootLoc.append(project.getOutputLocation());
      }
      catch (JavaModelException e) {
         FavoritesLog.logError(e);
         return null;
      }
      for (int i = 0; i < entries.length; i++) {
         IClasspathEntry entry = entries[i];
         switch (entry.getEntryKind()) {

         case IClasspathEntry.CPE_SOURCE:
            IPath path = entry.getOutputLocation();
            if (path != null)
               path = rootLoc.append(path);
            else
               path = outputLocation;
            path = path.append(relativePathToClassFile);
            byte[] buf = readBytes(path.toFile());
            if (buf != null)
               return buf;
            break;

         case IClasspathEntry.CPE_LIBRARY:
         case IClasspathEntry.CPE_PROJECT:
            // Handle other entry types here.
            break;

         default:
            break;
         }
      }
      return null;
   }

   /**
    * Read bytes from the specified file.
    * 
    * @param file the file to be read
    * @return the bytes from the file or null if the file is
    *         <code>null</code>, does not exist, or could not be
    *         read
    */
   private static byte[] readBytes(File file) {
      if (file == null || !file.exists())
         return null;
      InputStream stream = null;
      try {
         stream = new BufferedInputStream(new FileInputStream(file));
         int size = 0;
         byte[] buf = new byte[10];
         while (true) {
            int count = stream.read(buf, size, buf.length - size);
            if (count < 0)
               break;
            size += count;
            if (size < buf.length)
               break;
            byte[] newBuf = new byte[size + 10];
            System.arraycopy(buf, 0, newBuf, 0, size);
            buf = newBuf;
         }
         byte[] result = new byte[size];
         System.arraycopy(buf, 0, result, 0, size);
         return result;
      }
      catch (Exception e) {
         FavoritesLog.logError(e);
         return null;
      }
      finally {
         try {
            if (stream != null)
               stream.close();
         }
         catch (IOException e) {
            FavoritesLog.logError(e);
            return null;
         }
      }
   }
}
