package com.qualityeclipse.favorites.views;

import java.util.Comparator;
import java.util.Iterator;

import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ColumnPixelData;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.ViewPart;

import com.qualityeclipse.favorites.actions.CopyFavoritesAction;
import com.qualityeclipse.favorites.actions.CutFavoritesAction;
import com.qualityeclipse.favorites.actions.FavoritesViewFilterAction;
import com.qualityeclipse.favorites.actions.PasteFavoritesAction;
import com.qualityeclipse.favorites.actions.RemoveFavoritesAction;
import com.qualityeclipse.favorites.actions.RenameFavoriteAction;
import com.qualityeclipse.favorites.model.FavoritesManager;
import com.qualityeclipse.favorites.model.IFavoriteItem;
import com.qualityeclipse.favorites.util.AutoResizeTableLayout;

/**
 * This is the primary class behind the Favorites view. It creates the
 * view controls, associates those controls with a model, defines how
 * the model content should be displayed, etc.
 */
public class FavoritesView extends ViewPart {
   
   public static final String ID = 
      "com.qualityeclipse.favorites.views.FavoritesView";

   private TableColumn typeColumn;
   private TableColumn nameColumn;
   private TableColumn locationColumn;
   
   private TableViewer viewer;
   private FavoritesViewSorter sorter;   

   private CutFavoritesAction cutAction;
   private CopyFavoritesAction copyAction;
   private PasteFavoritesAction pasteAction;
   private RemoveFavoritesAction removeAction;   
   private FavoritesViewFilterAction filterAction; 
   private RenameFavoriteAction renameAction;
   
   private Clipboard clipboard;
   private ISelectionListener pageSelectionListener;
   
   private IMemento memento;
   
   /**
    * The constructor.
    */
   public FavoritesView() {
   }

   /**
    * This is a callback that will allow us to create the viewer
    * and initialize it.
    */
   public void createPartControl(Composite parent) {
      createTableViewer(parent);
      createTableSorter();
      createActions();
      createContextMenu();
      createToolbarButtons();
      createViewPulldownMenu();
      hookKeyboardActions();
      hookGlobalActions();
      hookDragAndDrop();
      hookMouse();
      hookPageSelection();
   }

   private void createTableViewer(Composite parent) {
      viewer = new TableViewer(parent, 
            SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI | SWT.FULL_SELECTION);
      final Table table = viewer.getTable();
      AutoResizeTableLayout layout = new AutoResizeTableLayout(table);
      table.setLayout(layout);

      typeColumn = new TableColumn(table, SWT.LEFT);
      typeColumn.setText("");
      // typeColumn.setWidth(18);
      layout.addColumnData(new ColumnPixelData(18));
      
      nameColumn = new TableColumn(table, SWT.LEFT);
      nameColumn.setText("Name");
      // nameColumn.setWidth(200);
      layout.addColumnData(new ColumnWeightData(200));

      locationColumn = new TableColumn(table, SWT.LEFT);
      locationColumn.setText("Location");
      // locationColumn.setWidth(450);
      layout.addColumnData(new ColumnWeightData(200));

      table.setHeaderVisible(true);
      table.setLinesVisible(false);

      viewer.setContentProvider(new FavoritesViewContentProvider());
      viewer.setLabelProvider(new FavoritesViewLabelProvider());
      viewer.setInput(FavoritesManager.getManager());

      getSite().setSelectionProvider(viewer);
   }
   
   private void createTableSorter() {
      Comparator nameComparator = new Comparator() {
         public int compare(Object o1, Object o2) {
            return ((IFavoriteItem) o1)
               .getName()
               .compareTo(
               ((IFavoriteItem) o2).getName());
         }
      };
      Comparator locationComparator = new Comparator() {
         public int compare(Object o1, Object o2) {
            return ((IFavoriteItem) o1)
               .getLocation()
               .compareTo(
               ((IFavoriteItem) o2).getLocation());
         }
      };
      Comparator typeComparator = new Comparator() {
         public int compare(Object o1, Object o2) {
            return ((IFavoriteItem) o1)
               .getType()
               .compareTo(
               ((IFavoriteItem) o2).getType());
         }
      };
      sorter = new FavoritesViewSorter(
         viewer,
         new TableColumn[] {
            nameColumn, locationColumn, typeColumn },
         new Comparator[] {
            nameComparator, locationComparator, typeComparator
      });
      if (memento != null)
         sorter.init(memento);
      viewer.setSorter(sorter);
   }
   
   private void createActions() {
      IWorkbench workbench = PlatformUI.getWorkbench();
      ISharedImages platformImages = workbench.getSharedImages();

      copyAction = new CopyFavoritesAction(this, "Copy");
      copyAction.setImageDescriptor(platformImages
         .getImageDescriptor(ISharedImages.IMG_TOOL_COPY));
      copyAction.setDisabledImageDescriptor(platformImages
         .getImageDescriptor(ISharedImages.IMG_TOOL_COPY_DISABLED));
      copyAction.setToolTipText("Copy the selected favorite items");

      removeAction = new RemoveFavoritesAction(this, "Remove");
      removeAction.setImageDescriptor(platformImages
         .getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
      removeAction.setDisabledImageDescriptor(platformImages
         .getImageDescriptor(ISharedImages.IMG_TOOL_DELETE_DISABLED));
      removeAction
         .setToolTipText("Remove the selected favorite items");

      pasteAction = new PasteFavoritesAction(this, "Paste");
      pasteAction.setImageDescriptor(platformImages
         .getImageDescriptor(ISharedImages.IMG_TOOL_PASTE));
      pasteAction.setDisabledImageDescriptor(platformImages
         .getImageDescriptor(ISharedImages.IMG_TOOL_PASTE_DISABLED));
      pasteAction
         .setToolTipText("Paste the selected favorite items");
      
      cutAction = new CutFavoritesAction(copyAction, removeAction,"Cut");
      cutAction.setImageDescriptor(platformImages
         .getImageDescriptor(ISharedImages.IMG_TOOL_CUT));
      cutAction.setDisabledImageDescriptor(platformImages
         .getImageDescriptor(ISharedImages.IMG_TOOL_CUT_DISABLED));
      cutAction
         .setToolTipText("Cut the selected favorite items");
      
      renameAction = new RenameFavoriteAction(
         this, viewer.getTable(), "Rename");
   }
   
   private void createContextMenu() {
      MenuManager menuMgr = new MenuManager("#PopupMenu");
      menuMgr.setRemoveAllWhenShown(true);
      menuMgr.addMenuListener(new IMenuListener() {
         public void menuAboutToShow(IMenuManager m) {
            FavoritesView.this.fillContextMenu(m);
         }
      });
      Menu menu = menuMgr.createContextMenu(viewer.getControl());
      viewer.getControl().setMenu(menu);
      getSite().registerContextMenu(menuMgr, viewer);
   }
   
   private void fillContextMenu(IMenuManager menuMgr) {
      boolean isEmpty = viewer.getSelection().isEmpty();
      cutAction.setEnabled(!isEmpty);
      menuMgr.add(cutAction);
      copyAction.setEnabled(!isEmpty);
      menuMgr.add(copyAction);
      menuMgr.add(pasteAction);
      menuMgr.add(new Separator());
      menuMgr.add(renameAction);
      menuMgr.add(new Separator());
      removeAction.setEnabled(!isEmpty);
      menuMgr.add(removeAction);
      menuMgr.add(new Separator(
            IWorkbenchActionConstants.MB_ADDITIONS));
   }
   
   private void createToolbarButtons() {
      IToolBarManager toolBarMgr = getViewSite().getActionBars().getToolBarManager();
      toolBarMgr.add(cutAction);
      cutAction.setEnabled(false);
      toolBarMgr.add(copyAction);
      copyAction.setEnabled(false);
      toolBarMgr.add(pasteAction);
      toolBarMgr.add(removeAction);
      removeAction.setEnabled(false);
      viewer.addSelectionChangedListener(
         new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
               boolean isEmpty = !event.getSelection().isEmpty();
               cutAction.setEnabled(isEmpty);
               copyAction.setEnabled(isEmpty);
               removeAction.setEnabled(isEmpty);
            }
      });
   }
   
   private void createViewPulldownMenu() {
      IMenuManager menu =
         getViewSite().getActionBars().getMenuManager();
      filterAction =
         new FavoritesViewFilterAction(viewer, "Filter...");
      if (memento != null)
         filterAction.init(memento);
      menu.add(filterAction);
   }
   
   private void hookKeyboardActions() {
      viewer.getControl().addKeyListener(new KeyAdapter() {
         public void keyReleased(KeyEvent event) {
            handleKeyReleased(event);
         }
      });
   }

   private void hookGlobalActions() {
      getViewSite().getActionBars().setGlobalActionHandler(
         ActionFactory.CUT.getId(), cutAction);
      getViewSite().getActionBars().setGlobalActionHandler(
         ActionFactory.COPY.getId(), copyAction);
      getViewSite().getActionBars().setGlobalActionHandler(
         ActionFactory.PASTE.getId(), pasteAction);
      getViewSite().getActionBars().setGlobalActionHandler(
         ActionFactory.DELETE.getId(), removeAction);
   }
   
   private void hookDragAndDrop() {
      new FavoritesDragSource(this, viewer);
      new FavoritesDropTarget(this, viewer);
   }
   
   private void hookMouse() {
      viewer.getTable().addMouseListener(new MouseAdapter() {
         public void mouseUp(MouseEvent e) {
            if ((e.stateMask & SWT.ALT) != 0)
               renameAction.run();
         }
         public void mouseDoubleClick(MouseEvent e) {
            com.qualityeclipse.favorites.actions.OpenEditorActionDelegate
               .openEditor(getSite().getPage(), viewer.getSelection());
         }
      });
   }
   
   private void hookPageSelection() {
      pageSelectionListener = new ISelectionListener() {
         public void selectionChanged(
            IWorkbenchPart part,
            ISelection selection) {
               pageSelectionChanged(part, selection);
         }
      };
      getSite().getPage().addPostSelectionListener(
         pageSelectionListener);
   }

   protected void pageSelectionChanged(
      IWorkbenchPart part,
      ISelection selection
   ) {
      if (part == this)
         return;
      if (!(selection instanceof IStructuredSelection))
         return;
      IStructuredSelection sel = (IStructuredSelection) selection;
      IFavoriteItem[] items = FavoritesManager.getManager()
         .existingFavoritesFor(sel.iterator());
      if (items.length > 0)
         viewer.setSelection(new StructuredSelection(items), true);
   }
   
   protected void handleKeyReleased(KeyEvent event) {
      if (event.keyCode == SWT.F2 && event.stateMask == 0) {
         renameAction.run();
      }
      if (event.character == SWT.DEL && event.stateMask == 0) {
         removeAction.run();
      }
   }
   
   /**
    * Passing the focus request to the viewer's control.
    */
   public void setFocus() {
      viewer.getControl().setFocus();
   }

   /**
    * For testing purposes only.
    * @return the table viewer in the favorites view 
    */
   public TableViewer getFavoritesViewer() {
      return viewer;
   }

   /**
    * Answer the currently selected favorite items
    * @return an Array of IFavoriteItem instances
    *    (not <code>null</code>, but may be empty)
    */
   public IFavoriteItem[] getSelectedFavorites() {
      IStructuredSelection selection = 
         (IStructuredSelection) viewer.getSelection();
      IFavoriteItem[] items = new IFavoriteItem[selection.size()];
      Iterator iter = selection.iterator();
      int index = 0;
      while (iter.hasNext())
         items[index++] = (IFavoriteItem) iter.next();
      return items;
   }
   
   /**
    * Lazily initialize and answer the clipboard
    * @return the clipboard (not <code>null</code>)
    */
   public Clipboard getClipboard() {
      if (clipboard == null)
         clipboard = new Clipboard(getSite().getShell().getDisplay());
      return clipboard;
   }

   /**
    * Disposes of this view. Clients should not call this method; it
    * is called by the Eclipse infrastructure.
    * <p>
    * This is the last method called on the view. At this point the
    * part controls (if they were ever created) have been disposed as
    * part of an SWT composite. There is no guarantee that
    * createPartControl() has been called, so the part controls may
    * never have been created.
    * <p>
    * Within this method a part may release any resources, fonts,
    * images, etc.&nbsp; held by this part. It is also very important
    * to deregister all listeners from the workbench.
    */
   public void dispose() {
      if (clipboard != null)
         clipboard.dispose();
      if (pageSelectionListener != null)
         getSite().getPage().removePostSelectionListener(
             pageSelectionListener);
      super.dispose();
   }
   
   /**
    * Saves the view state such as sort order and filter state 
    * within a memento.
    */
   public void saveState(IMemento memento) {
      super.saveState(memento);
      sorter.saveState(memento);
      filterAction.saveState(memento);
   }
   
   /**
    * Initializes this view with the given view site. A memento is
    * passed to the view which contains a snapshot of the views state
    * such as sort order and filter state from a previous session.
    * In our case, the sort and filter state cannot be initialized immediately,
    * so we cache the memento for later.
    */
   public void init(IViewSite site, IMemento memento)
         throws PartInitException 
   {
      super.init(site, memento);
      this.memento = memento;
   }
}