package com.qualityeclipse.favorites.views;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Display;

import com.qualityeclipse.favorites.model.FavoritesManager;
import com.qualityeclipse.favorites.model.FavoritesManagerEvent;
import com.qualityeclipse.favorites.model.FavoritesManagerListener;

/**
 * The content provider is responsible for providing objects to the
 * view. It can wrap existing objects in adapters or simply return
 * objects as-is. These objects may be sensitive to the current input
 * of the view, or ignore it and always show the same content (like
 * Task List, for example).
 * <p>
 * In our case, it serves up IFavoriteItem instances that are then
 * displayed, one per row, in the Favorites view. In addition, it is
 * responsible for updating the viewer when the content of
 * FavoritesManager changes.
 */
class FavoritesViewContentProvider 
   implements IStructuredContentProvider, FavoritesManagerListener
{
   private TableViewer viewer;
   private FavoritesManager manager;

   /**
    * Notifies this content provider that the given viewer's input
    * has been switched to a different element.
    */
   public void inputChanged(
      Viewer viewer, Object oldInput, Object newInput
   ) {
      this.viewer = (TableViewer) viewer;
      if (manager != null)
         manager.removeFavoritesManagerListener(this);
      manager = (FavoritesManager) newInput;
      if (manager != null)
         manager.addFavoritesManagerListener(this);
   }

   public void dispose() {
   }

   /**
    * Returns the elements to display in the viewer when its input is
    * set to the given element. In our case, the partent is assumed to
    * be the FavoritesManager and the items returned, the Favorite
    * items in that manager.
    */
   public Object[] getElements(Object parent) {
      return manager.getFavorites();
   }

   /**
    * Called when one or more favorites items 
    * have been added and/or removed.
    */
   public void favoritesChanged(final FavoritesManagerEvent event) {
      
      // If this is the UI thread, then make the change
      if (Display.getCurrent() != null) {
         updateViewer(event);
         return;
      }
   
      // otherwise redirect to execute on the UI thread
      Display.getDefault().asyncExec(new Runnable() {
         public void run() {
            updateViewer(event);
         }
      });
   }
   
   private void updateViewer(final FavoritesManagerEvent event) {
      // Use the setRedraw method to reduce flicker
      // when adding or removing multiple items in a table.
      viewer.getTable().setRedraw(false);
      try {
         viewer.remove(event.getItemsRemoved());
         viewer.add(event.getItemsAdded());
      }
      finally {
         viewer.getTable().setRedraw(true);
      }
   }
}